from flask_login import UserMixin,login_user,login_required,logout_user,LoginManager,current_user
from datetime import datetime
from __init__ import db
import enum, re, random, pytz, uuid
from sqlalchemy.orm import backref
tz = pytz.timezone('Africa/Nairobi')

def SlugifyCaps(string):
	string = re.sub('[^A-Za-z0-9]+', ' ', string).replace(" ", "_").upper()
	return string

def Slugify(string):
	string = re.sub('[^A-Za-z0-9]+', ' ', string).replace(" ", "_").lower()
	return string

class Users(UserMixin, db.Model):
    __tablename__ = 'users'
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    email = db.Column(db.String(80),unique=True)
    username = db.Column(db.String(80),unique=True)
    pwd = db.Column(db.String(300), nullable=False, unique=True)
    authenticated = db.Column(db.Boolean, default=False)
    roles = db.Column(db.String(100), default = 'user') 
    
    def __repr__(self):
        return '<User %r>' % self.username
    
class Testimonials(db.Model):
    __tablename__ = "testimonials"
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    message = db.Column(db.Text)
    image = db.Column(db.String(255))
    upload_date = db.Column(db.DateTime, default=datetime.now())
    update_date = db.Column(db.DateTime)
    uploader = db.Column(db.Integer)
    published = db.Column(db.Boolean, default=False)

    def __repr__(self):
        return '<Testimonial %r>' % self.name
    
class Partners(db.Model):
    __tablename__ = "partners"
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    description = db.Column(db.Text)
    image = db.Column(db.String(255))
    upload_date = db.Column(db.DateTime, default=datetime.now())
    update_date = db.Column(db.DateTime)
    uploader = db.Column(db.Integer)
    published = db.Column(db.Boolean, default=False)

    def __repr__(self):
        return '<Testimonial %r>' % self.name
    
class Leaders(db.Model):
    __tablename__ = "leaders"
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    phone = db.Column(db.String(255))
    email = db.Column(db.String(255))
    post = db.Column(db.String(255))
    message = db.Column(db.Text)
    
    image = db.Column(db.String(255))
    
    facebook = db.Column(db.String(255), default=None)
    linkedIn = db.Column(db.String(255), default=None)
    twitter = db.Column(db.String(255), default=None)
    
    upload_date = db.Column(db.DateTime, default=datetime.now())
    active = db.Column(db.Boolean, default=False)

    def __repr__(self):
        return '<Testimonial %r>' % self.name
    
class Inquiries(db.Model):
    __tablename__ = "inquiries"
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    email = db.Column(db.String(80))
    phone = db.Column(db.String(80))
    message = db.Column(db.Text)
    upload_date = db.Column(db.DateTime, default=datetime.now(tz))
    update_date = db.Column(db.DateTime)
    
    service = db.Column(db.Integer)
    handled = db.Column(db.Boolean, default=False)
    
    def __repr__(self):
        return '<Inquiries %r>' % self.id
    
class Subscribers(db.Model):
    __tablename__ = "subscribers"
    
    id = db.Column(db.Integer, primary_key = True)
    email = db.Column(db.String(255))
    active = db.Column(db.Boolean, default=True)
    upload_date = db.Column(db.DateTime, default=datetime.now())
    
    def __repr__(self):
        return '<Subscribers %r>' % self.id
    
class ServiceCategories(db.Model):
    __tablename__ = "service_categories"
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    slug = db.Column(db.String(255),unique=True)
    icon = db.Column(db.String(255))
    description = db.Column(db.Text)
    active = db.Column(db.Boolean, default=True)
    upload_date = db.Column(db.DateTime, default=datetime.now())
    
    def __repr__(self):
        return '<ServiceCategories %r>' % self.id

    def __setattr__(self, key, value):
        super(ServiceCategories, self).__setattr__(key, value)
        if key == 'name':
            self.slug = Slugify(self.name)
            
class Projects(db.Model):
    __tablename__ = "projects"
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    slug = db.Column(db.String(255),unique=True)
    icon = db.Column(db.String(255))
    description = db.Column(db.Text)
    active = db.Column(db.Boolean, default=True)
    images = db.Column(db.PickleType, default=[])
    upload_date = db.Column(db.DateTime, default=datetime.now())
    
    def __repr__(self):
        return '<Projects %r>' % self.id

    def __setattr__(self, key, value):
        super(Projects, self).__setattr__(key, value)
        if key == 'name':
            self.slug = Slugify(self.name)
            
class Activity(db.Model):
    __tablename__ = "activity"
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    slug = db.Column(db.String(255),unique=True)
    icon = db.Column(db.String(255))
    description = db.Column(db.Text)
    active = db.Column(db.Boolean, default=True)
    images = db.Column(db.PickleType, default=[])
    upload_date = db.Column(db.DateTime, default=datetime.now())
    
    def __repr__(self):
        return '<Activity %r>' % self.id

    def __setattr__(self, key, value):
        super(Activity, self).__setattr__(key, value)
        if key == 'name':
            self.slug = Slugify(self.name)

class ActivityRequests(db.Model):
    __tablename__ = "activity_requests"
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    email = db.Column(db.String(255))
    phone = db.Column(db.String(255))
    
    slug = db.Column(db.String(255), unique=True)
    activity = db.Column(db.Integer)
    active = db.Column(db.Boolean, default=True)
    date = db.Column(db.DateTime, default=datetime.now())
    
    def __repr__(self):
        return '<Activity Requests %r>' % self.id

    def __setattr__(self, key, value):
        super(ActivityRequests, self).__setattr__(key, value)
        if key == 'name':
            self.slug = f"{Slugify(self.name)}_{random.randint(0,1000)}"

            
class AllServices(db.Model):
    __tablename__ = "all_services"
    
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    slug = db.Column(db.String(255),unique=True)
    icon = db.Column(db.String(255))
    description = db.Column(db.Text)
    service_category = db.Column(db.Integer)
    active = db.Column(db.Boolean, default=True)
    upload_date = db.Column(db.DateTime, default=datetime.now())
    
    def __repr__(self):
        return '<AllServices %r>' % self.id

    def __setattr__(self, key, value):
        super(AllServices, self).__setattr__(key, value)
        if key == 'name':
            self.slug = Slugify(self.name)
    
class CareersModel(db.Model):
    __tablename__ = "career_model"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(255))
    email = db.Column(db.String(80))
    phone = db.Column(db.String(80))
    message = db.Column(db.Text)
    document = db.Column(db.String(255))
    active = db.Column(db.Boolean, default=True)
    upload_date = db.Column(db.DateTime, default=datetime.now(tz))
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.name})'
    
class Posts(db.Model):
    __tablename__ = "posts"
    
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(255), nullable=False)
    slug = db.Column(db.String(255), nullable=False, unique=True)
    
    image = db.Column(db.String(255))
    upload_date = db.Column(db.DateTime, nullable=False, default=datetime.now())
    content = db.Column(db.Text, nullable=False)
    keywords = db.Column(db.PickleType(), default=[])
    user_id = db.Column(db.Integer, nullable=False)
    active = db.Column(db.Boolean, default=True)
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.title})'
    def __setattr__(self, key, value):
        super(Posts, self).__setattr__(key, value)
        if key == 'title':
            self.slug = f"{Slugify(self.title)}_{self.id}"
    
class DraftPosts(db.Model):
    __tablename__ = "draft_posts"
    
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(120))
    slug = db.Column(db.String(255), nullable=False)
    
    image = db.Column(db.String(255))
    upload_date = db.Column(db.DateTime, default=datetime.now())
    content = db.Column(db.Text)
    keywords = db.Column(db.PickleType(), default=[])
    user_id = db.Column(db.Integer)
    active = db.Column(db.Boolean, default=True)
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.title})'
    
    def __setattr__(self, key, value):
        super(DraftPosts, self).__setattr__(key, value)
        if key == 'title':
            self.slug = Slugify(self.title)
    

class Products(db.Model):
    __tablename__ = "products"
    
    id = db.Column(db.Integer, primary_key=True)
    p_id = db.Column(db.String(100))
    name = db.Column(db.String(300))
    currency = db.Column(db.String(4), default='KES')
    price = db.Column(db.Integer)
    sold = db.Column(db.Integer, default=0)
    filesize = db.Column(db.Integer)
    link = db.Column(db.String(300))
    cover = db.Column(db.String(300))
    upload_date = db.Column(db.DateTime, default=datetime.now())
    description = db.Column(db.Text)
    active = db.Column(db.Boolean, default=False)
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.name})'
    
    def __setattr__(self, key, value):
        super(Products, self).__setattr__(key, value)
        if key == 'name':
            self.p_id = SlugifyCaps(str(uuid.uuid4()))
    
class Carts(db.Model):
    __tablename__ = "carts"
    
    id = db.Column(db.Integer, primary_key=True)
    
    items = db.Column(db.PickleType(), default=[])
    checkedout = db.Column(db.Boolean, default = False)
    cost = db.Column(db.Float, default=0.00)
    
    user = db.Column(db.Integer)
    start_date = db.Column(db.DateTime, default=datetime.now())
    checkout_date = db.Column(db.DateTime)
    
    
    
class Country(db.Model):
    __tablename__ = "countries"
    
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(60))
    iso = db.Column(db.String(2))
    iso3 = db.Column(db.String(3))
    dial = db.Column(db.String(5))
    currency = db.Column(db.String(3))
    currency_name = db.Column(db.String(60))
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.name})'
    

class Donations(db.Model):
    __tablename__ = 'donations'
    
    id = db.Column(db.Integer, primary_key = True) 
    name = db.Column(db.String(255), nullable=False)
    email = db.Column(db.String(255), nullable=False)
    phone = db.Column(db.String(50), nullable=False)
    amount = db.Column(db.Integer, nullable=False)
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.name})'
    
class MpesaAccessToken(db.Model):
    __tablename__ = 'access_token'
    id = db.Column(db.Integer, primary_key = True) 
    token = db.Column(db.String(255), nullable=False)
    date = db.Column(db.DateTime, default=datetime.now)
    expiry = db.Column(db.Integer, nullable=False)
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.token})'
    
class PaymentsTable(db.Model):
    id = db.Column(db.Integer, primary_key = True) 
    payment_type = db.Column(db.String(255), nullable=False)
    email = db.Column(db.String(255))
    user_id = db.Column(db.Integer)
    
    checkoutid = db.Column(db.String(255))
    merchantid = db.Column(db.String(255))
    resultcode = db.Column(db.String(255))
    resultdesc = db.Column(db.String(255))
    
    amount = db.Column(db.Integer) 
    currency = db.Column(db.String(50), default = "KES")
    receipt_number = db.Column(db.String(255))
    trasaction_date = db.Column(db.DateTime, default=datetime.now)
    phone_number = db.Column(db.String(255))
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.payment_type})'
    
    
class MpesaPaymentsRequests(db.Model):
    __tablename__ = 'mpesa_payments_requests'
    id = db.Column(db.Integer, primary_key = True) 
    email = db.Column(db.String(255))
    user_id = db.Column(db.Integer)
    
    CheckoutRequestID = db.Column(db.String(255))
    CustomerMessage = db.Column(db.String(255))
    MerchantRequestID = db.Column(db.String(255))
    ResponseCode = db.Column(db.String(255))
    ResponseDescription = db.Column(db.String(255))
    
    trasaction_date = db.Column(db.DateTime, default=datetime.now)
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.name})'
    
class MpesaErrors(db.Model):
    __tablename__ = 'mpesa_payments_errors'
    
    id = db.Column(db.Integer, primary_key = True) 
    user_id = db.Column(db.Integer)
    
    checkoutid = db.Column(db.String(255))
    merchantid = db.Column(db.String(255))
    resultcode = db.Column(db.String(255))
    resultdesc = db.Column(db.String(255))
    
    date = db.Column(db.DateTime, default=datetime.now)
    
    def __repr__(self):
        return f'{self.__class__.__name__}({self.id}, {self.checkoutid})'
    
    